"""Asynchronous client for the PVOutput API."""
from __future__ import annotations

from datetime import date, datetime, time
from typing import Optional, Union

from pydantic import BaseModel, validator


class Status(BaseModel):
    """Object holding the latest status information and live output data."""

    reported_date: date
    reported_time: time

    energy_consumption: Optional[int]
    energy_generation: Optional[int]
    normalized_output: Optional[float]
    power_consumption: Optional[int]
    power_generation: Optional[int]
    temperature: Optional[float]
    voltage: Optional[float]

    @property
    def reported_datetime(self) -> datetime:
        """Return the timestamp of the status data.

        Returns:
            A datetime object.
        """
        return datetime.combine(self.reported_date, self.reported_time)

    @validator(
        "energy_consumption",
        "energy_generation",
        "normalized_output",
        "power_consumption",
        "power_generation",
        "temperature",
        "voltage",
        pre=True,
    )
    @classmethod
    def filter_not_a_number(
        cls, value: Union[str, int, float]  # noqa: F841
    ) -> Optional[Union[str, int, float]]:
        """Filter out NaN values.

        Args:
            value: Value to filter.

        Returns:
            Filtered value.
        """
        if value == "NaN":
            return None
        return value

    @validator("reported_date", pre=True)
    @classmethod
    def preparse_date(cls, value: str) -> str:  # noqa: F841
        """Preparse date so Pydantic understands it.

        Args:
            value: Date value to preparse.

        Returns:
            Preparsed date value.
        """
        return f"{value[:4]}-{value[4:6]}-{value[6:]}"


class System(BaseModel):
    """Object holding the latest system information."""

    array_tilt: Optional[float]
    install_date: Optional[date]
    inverter_brand: Optional[str]
    inverter_power: Optional[int]
    inverters: Optional[int]
    latitude: Optional[float]
    longitude: Optional[float]
    orientation: Optional[str]
    panel_brand: Optional[str]
    panel_power: Optional[int]
    panels: Optional[int]
    shade: Optional[str]
    status_interval: Optional[int]
    system_name: str
    system_size: Optional[int]
    zipcode: Optional[int]

    @validator("install_date", pre=True)
    @classmethod
    def preparse_date(cls, value: str) -> str:  # noqa: F841
        """Preparse date so Pydantic understands it.

        Args:
            value: Date value to preparse.

        Returns:
            Preparsed date value.
        """
        return f"{value[:4]}-{value[4:6]}-{value[6:]}"



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['pvo']

package_data = \
{'': ['*']}

install_requires = \
['aiohttp>=3.0.0', 'pydantic>=1.8.0,<2.0.0', 'yarl>=1.6.0']

setup_kwargs = {
    'name': 'pvo',
    'version': '0.2.0',
    'description': 'Asynchronous client for the PVOutput API.',
    'long_description': '# Python: Asynchronous client for the PVOutput API\n\n[![GitHub Release][releases-shield]][releases]\n[![Python Versions][python-versions-shield]][pypi]\n![Project Stage][project-stage-shield]\n![Project Maintenance][maintenance-shield]\n[![License][license-shield]](LICENSE.md)\n\n[![Build Status][build-shield]][build]\n[![Code Coverage][codecov-shield]][codecov]\n[![Code Quality][code-quality-shield]][code-quality]\n\n[![Sponsor Frenck via GitHub Sponsors][github-sponsors-shield]][github-sponsors]\n\n[![Support Frenck on Patreon][patreon-shield]][patreon]\n\nAsynchronous client for the PVOutput API.\n\n## About\n\nThis package allows you to monitor PV systems connected to [PVOutput][pvoutput]\nprogrammatically. It is mainly created to allow third-party programs to\nintegrate with PVOutput.\n\n## Installation\n\n```bash\npip install pvo\n```\n\n## Usage\n\n```python\nimport asyncio\n\nfrom pvo import PVOutput\n\n\nasync def main():\n    """Show example on using the PVOutput API client."""\n    async with PVOutput(\n        api_key="API_KEY_FROM_PVOUTPUT_ORG",\n        system_id=60017,\n    ) as pvoutput:\n\n        status = await pvoutput.status()\n        print(status)\n\n\nif __name__ == "__main__":\n    asyncio.run(main())\n```\n\n## Changelog & Releases\n\nThis repository keeps a change log using [GitHub\'s releases][releases]\nfunctionality. The format of the log is based on\n[Keep a Changelog][keepchangelog].\n\nReleases are based on [Semantic Versioning][semver], and use the format\nof `MAJOR.MINOR.PATCH`. In a nutshell, the version will be incremented\nbased on the following:\n\n- `MAJOR`: Incompatible or major changes.\n- `MINOR`: Backwards-compatible new features and enhancements.\n- `PATCH`: Backwards-compatible bugfixes and package updates.\n\n## Contributing\n\nThis is an active open-source project. We are always open to people who want to\nuse the code or contribute to it.\n\nWe\'ve set up a separate document for our\n[contribution guidelines](CONTRIBUTING.md).\n\nThank you for being involved! :heart_eyes:\n\n## Setting up development environment\n\nThis Python project is fully managed using the [Poetry][poetry] dependency\nmanager. But also relies on the use of NodeJS for certain checks during\ndevelopment.\n\nYou need at least:\n\n- Python 3.8+\n- [Poetry][poetry-install]\n- NodeJS 14+ (including NPM)\n\nTo install all packages, including all development requirements:\n\n```bash\nnpm install\npoetry install\n```\n\nAs this repository uses the [pre-commit][pre-commit] framework, all changes\nare linted and tested with each commit. You can run all checks and tests\nmanually, using the following command:\n\n```bash\npoetry run pre-commit run --all-files\n```\n\nTo run just the Python tests:\n\n```bash\npoetry run pytest\n```\n\n## Authors & contributors\n\nThe original setup of this repository is by [Franck Nijhof][frenck].\n\nFor a full list of all authors and contributors,\ncheck [the contributor\'s page][contributors].\n\n## License\n\nMIT License\n\nCopyright (c) 2021 Franck Nijhof\n\nPermission is hereby granted, free of charge, to any person obtaining a copy\nof this software and associated documentation files (the "Software"), to deal\nin the Software without restriction, including without limitation the rights\nto use, copy, modify, merge, publish, distribute, sublicense, and/or sell\ncopies of the Software, and to permit persons to whom the Software is\nfurnished to do so, subject to the following conditions:\n\nThe above copyright notice and this permission notice shall be included in all\ncopies or substantial portions of the Software.\n\nTHE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR\nIMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,\nFITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE\nAUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER\nLIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,\nOUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE\nSOFTWARE.\n\n[build-shield]: https://github.com/frenck/python-pvoutput/actions/workflows/tests.yaml/badge.svg\n[build]: https://github.com/frenck/python-pvoutput/actions/workflows/tests.yaml\n[code-quality-shield]: https://img.shields.io/lgtm/grade/python/g/frenck/python-pvoutput.svg?logo=lgtm&logoWidth=18\n[code-quality]: https://lgtm.com/projects/g/frenck/python-pvoutput/context:python\n[codecov-shield]: https://codecov.io/gh/frenck/python-pvoutput/branch/master/graph/badge.svg\n[codecov]: https://codecov.io/gh/frenck/python-pvoutput\n[contributors]: https://github.com/frenck/python-pvoutput/graphs/contributors\n[frenck]: https://github.com/frenck\n[github-sponsors-shield]: https://frenck.dev/wp-content/uploads/2019/12/github_sponsor.png\n[github-sponsors]: https://github.com/sponsors/frenck\n[keepchangelog]: http://keepachangelog.com/en/1.0.0/\n[license-shield]: https://img.shields.io/github/license/frenck/python-pvoutput.svg\n[maintenance-shield]: https://img.shields.io/maintenance/yes/2021.svg\n[patreon-shield]: https://frenck.dev/wp-content/uploads/2019/12/patreon.png\n[patreon]: https://www.patreon.com/frenck\n[poetry-install]: https://python-poetry.org/docs/#installation\n[poetry]: https://python-poetry.org\n[pre-commit]: https://pre-commit.com/\n[project-stage-shield]: https://img.shields.io/badge/project%20stage-production%20ready-brightgreen.svg\n[pypi]: https://pypi.org/project/pvo/\n[python-versions-shield]: https://img.shields.io/pypi/pyversions/pvo\n[releases-shield]: https://img.shields.io/github/release/frenck/python-pvoutput.svg\n[releases]: https://github.com/frenck/python-pvoutput/releases\n[semver]: http://semver.org/spec/v2.0.0.html\n[pvoutput]: https://pvoutput.org/\n',
    'author': 'Franck Nijhof',
    'author_email': 'opensource@frenck.dev',
    'maintainer': 'Franck Nijhof',
    'maintainer_email': 'opensource@frenck.dev',
    'url': 'https://github.com/frenck/python-pvoutput',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""Asynchronous client for the PVOutput API."""
from __future__ import annotations

import asyncio
import socket
from dataclasses import dataclass
from importlib import metadata

import async_timeout
from aiohttp.client import ClientError, ClientResponseError, ClientSession
from aiohttp.hdrs import METH_GET
from yarl import URL

from .exceptions import (
    PVOutputAuthenticationError,
    PVOutputConnectionError,
    PVOutputError,
)
from .models import Status, System


@dataclass
class PVOutput:
    """Main class for handling connections with the PVOutput API."""

    api_key: str
    system_id: int

    request_timeout: float = 8.0
    session: ClientSession | None = None

    _close_session: bool = False

    async def _request(
        self,
        uri: str,
        *,
        method: str = METH_GET,
        data: dict | None = None,
    ) -> str:
        """Handle a request to the PVOutput API.

        A generic method for sending/handling HTTP requests done against
        the PVOutput API.

        Args:
            uri: Request URI, without '/service/r2/'.
            method: HTTP Method to use.
            data: Dictionary of parameters to send to the PVOutput API.

        Returns:
            The response body from the PVOutput API.

        Raises:
            PVOutputAuthenticationError: If the API key is invalid.
            PVOutputConnectionError: An error occurred while communicating with
                the PVOutput API.
            PVOutputError: Received an unexpected response from the PVOutput
                API.
        """
        version = metadata.version(__package__)
        url = URL("https://pvoutput.org/service/r2/").join(URL(uri))

        headers = {
            "Accept": "application/json",
            "User-Agent": f"PythonPVOutput/{version}",
            "X-Pvoutput-Apikey": self.api_key,
            "X-Pvoutput-SystemId": str(self.system_id),
        }

        if self.session is None:
            self.session = ClientSession()
            self._close_session = True

        try:
            async with async_timeout.timeout(self.request_timeout):
                response = await self.session.request(
                    method,
                    url,
                    data=data,
                    headers=headers,
                )
                response.raise_for_status()
        except asyncio.TimeoutError as exception:
            raise PVOutputConnectionError(
                "Timeout occurred while connecting to the PVOutput API"
            ) from exception
        except ClientResponseError as exception:
            if exception.status in [401, 403]:
                raise PVOutputAuthenticationError(
                    "Authentication to the PVOutput API failed"
                ) from exception
            raise PVOutputError(
                "Error occurred while connecting to the PVOutput API"
            ) from exception
        except (
            ClientError,
            socket.gaierror,
        ) as exception:
            raise PVOutputConnectionError(
                "Error occurred while communicating with the PVOutput API"
            ) from exception

        return await response.text()

    async def status(self) -> Status:
        """Retrieve system status information and live output data.

        Returns:
            An PVOutput Status object.
        """
        data = await self._request("getstatus.jsp")
        return Status.parse_obj(
            zip(
                [
                    "reported_date",
                    "reported_time",
                    "energy_generation",
                    "power_generation",
                    "energy_consumption",
                    "power_consumption",
                    "normalized_output",
                    "temperature",
                    "voltage",
                ],
                data.split(","),
            )
        )

    async def system(self) -> System:
        """Retrieve system information.

        Returns:
            An PVOutput System object.
        """
        data = await self._request("getsystem.jsp")
        return System.parse_obj(
            zip(
                [
                    "system_name",
                    "system_size",
                    "zipcode",
                    "panels",
                    "panel_power",
                    "panel_brand",
                    "inverters",
                    "inverter_power",
                    "inverter_brand",
                    "orientation",
                    "array_tilt",
                    "shade",
                    "install_date",
                    "latitude",
                    "longitude",
                    "status_interval",
                ],
                data.partition(";")[0].split(","),
            )
        )

    async def close(self) -> None:
        """Close open client session."""
        if self.session and self._close_session:
            await self.session.close()

    async def __aenter__(self) -> PVOutput:
        """Async enter.

        Returns:
            The PVOutput object.
        """
        return self

    async def __aexit__(self, *_exc_info) -> None:
        """Async exit.

        Args:
            _exc_info: Exec type.
        """
        await self.close()
